@@ -1,4 +1,4 @@
-__version__ = "0.0.4"
+__version__ = "0.0.6"
 __all__ = ["Date"]
 
 import re
@@ -9,7 +9,7 @@ import ply.yacc as yacc
 
 from stringtime.date import Date as stDate
 
-DEBUG = True
+DEBUG = False
 try:
     ERR_ICN = "\U0000274C"
     WARN_ICN = "\U000026A0"
@@ -65,9 +65,24 @@ tokens = (
     "DATE_END",
     "AM",
     "PM",
+    "A",
+    "COLON",
+    # "DATESTAMP",
 )
 
 
+def t_COLON(t):
+    r":"
+    return t
+
+
+def t_A(t):
+    r"\ba\b"
+    t.value = 1
+    t.type = "NUMBER"
+    return t
+
+
 def t_DATE_END(t):
     r"st\b|nd\b|rd\b|th\b"
     # print('date-end detected!', t.value)
@@ -95,13 +110,16 @@ def t_MINUS(t):
 def t_NUMBER(t):
     r"\d+"
     t.value = int(t.value)
-    # print('number detected!', t.value)
     return t
 
 
-# t_WORD_NUMBER = (
-#     r"one|two|three|four|five|six|seven|eight|nine|ten|eleven|twelve|thirteen|fourteen|fifteen|sixteen|seventeen|eighteen|nineteen|twenty|thirty|forty|fifty|sixty|seventy|eighty|ninety"
-# )
+# \d{4}-\d{2}-\d{2}|\d{4}-\d{2}|\d{4}|\d{2}-\d{2}
+# strings in the form: 2020-12-24 or 2020/12/24 or 2020|12|24
+# strings in the form: 2020-12 or 12/24 or 2020|12
+# def t_DATESTAMP(t):
+#     r"\d{4}-\d{2}-\d{2}|\d{4}-\d{2}|\d{4}|\d{2}-\d{2}|\d{4}/\d{2}/\d{2}|\d{4}/\d{2}|\d{4}|\d{2}/\d{2}|\d{4}|\d{2}|\d{2}|\d{4}|\d{2}|\d{4}|\d{2}|\d{2}"
+#     print('datestamp detected!', t.value)
+#     return t
 
 
 # TODO - test for all numbers
@@ -215,18 +233,14 @@ def t_BEFORE_YESTERDAY(t):
 
 t_TODAY = r"today"
 
-# t_AT = r"at|@"
-
 
 def t_AT(t):
     r"at|@"
-    # print('@ detected!', t.value)
     return t
 
 
 t_ON = r"on"
 t_OF = r"of"
-# t_THE = r"the"
 
 
 def t_AM(t):
@@ -243,7 +257,6 @@ def t_PM(t):
 
 def t_THE(t):
     r"the"
-    # print('the detected!', t.value)
     return t
 
 
@@ -277,9 +290,11 @@ class DateFactory:
     ):
         """creates a date with fixed props
 
+        unlike datatime it uses 0 indexing for months
+
         Args:
             year (_type_, optional): The year. Defaults to None.
-            month (_type_, optional): The month. Defaults to None.
+            month (_type_, optional): the month (0-11). Defaults to None.
             week (_type_, optional): _description_. Defaults to None.
             day (_type_, optional): _description_. Defaults to None.
             hour (_type_, optional): _description_. Defaults to None.
@@ -293,7 +308,7 @@ class DateFactory:
         if year is not None:
             d.set_year(year)
         if month is not None:
-            d.set_month(month)  # note - should this one be -1?
+            d.set_month(month)
         if week is not None:
             d.set_week(week)
         if day is not None:
@@ -332,11 +347,12 @@ class DateFactory:
         Returns:
             Date : Returns a Date object with the offsets applied.
         """
+        # print("Creating date!!", year, month, week, day, hour, minute, second)
         # TODO - should maybe optionally pass and remember the phrase on a new 'description' prop on Date...?
         d = stDate()
         if year is not None:
             stlog(
-                f"Creating new date {year} years from now: Current date:",
+                f"Increasing years by {year}",
                 str(d),
                 lvl="g",
             )
@@ -344,7 +360,7 @@ class DateFactory:
             d.set_fullyear(current_year + year)
         if month is not None:
             stlog(
-                f"Creating new date {month} months from now: Current date:",
+                f"Increasing months by {month}",
                 str(d),
             )
             currrent_month = d.get_month()
@@ -353,7 +369,7 @@ class DateFactory:
             )  # note the minus one is because Date expects 0-11 but humans say 1-12. wrong cos its the offset?
         if week is not None:
             stlog(
-                f"Creating new date {week} weeks from now: Current date:",
+                f"Increasing weeks by {week}",
                 str(d),
                 lvl="g",
             )
@@ -361,7 +377,7 @@ class DateFactory:
             d.set_date(currrent_day + week * 7)
         if day is not None:
             stlog(
-                f"Creating new date {day} days from now: Current date:",
+                f"Increasing days by {day}",
                 str(d),
                 lvl="g",
             )
@@ -369,7 +385,7 @@ class DateFactory:
             d.set_date(currrent_day + day)
         if hour is not None:
             stlog(
-                f"  - Creating new date {hour} hours from now: Current date:",
+                f"Increasing hours by {hour}",
                 str(d),
                 lvl="g",
             )
@@ -377,7 +393,7 @@ class DateFactory:
             d.set_hours(currrent_hour + hour)
         if minute is not None:
             stlog(
-                f"  - Creating new date {minute} minutes from now: Current date:",
+                f"Increasing minutes by {minute}",
                 str(d),
                 lvl="g",
             )
@@ -385,7 +401,7 @@ class DateFactory:
             d.set_minutes(currrent_minute + minute)
         if second is not None:
             stlog(
-                f"  - Creating new date {second} seconds from now: Current date:",
+                f"Increasing seconds by {second}",
                 str(d),
                 lvl="g",
             )
@@ -411,7 +427,7 @@ def p_date_object(p):
 
 
 def p_date_list(p):
-    "date_list :  date_list date"
+    "date_list : date_list date"
     p[0] = p[1] + [p[2]]
 
 
@@ -429,16 +445,70 @@ def p_date(p):
     date_list : date_or
     date_list : date_before_yesterday
     date_list : date_after_tomorrow
+    date_list : date_twice
+    date_list : timestamp
+    date_list : timestamp_adpt
     """
     p[0] = [p[1]]
 
 
+# def p_datestamp(p):
+#     """
+#     datestamp : NUMBER MINUS NUMBER
+#     datestamp : NUMBER MINUS NUMBER MINUS NUMBER
+#     """
+#     if len(p) == 4:
+#         p[0] = stDate.create_date_with_offsets(
+#             year=p[1], month=p[2], day=p[3]
+#         )
+
+
+def p_timestamp(p):
+    """
+    timestamp : NUMBER COLON NUMBER
+    timestamp : NUMBER COLON NUMBER COLON NUMBER
+    """
+    if len(p) == 4:
+        params = {"hour": p[1], "minute": p[3], "second": 0}
+    elif len(p) == 6:
+        params = {"hour": p[1], "minute": p[3], "second": p[5]}
+    p[0] = DateFactory.create_date(**params)
+
+
+# saves having multiple redefinitions inside timestamp
+def p_timestamp_adapter(p):
+    """
+    timestamp_adpt : timestamp AM
+    timestamp_adpt : timestamp PM
+    timestamp_adpt : AT timestamp
+    timestamp_adpt : AT timestamp PM
+    timestamp_adpt : AT timestamp AM
+    """
+    if len(p) == 3:
+        if p[1] == "at":
+            p[0] = p[2]
+        else:
+            if p[2] == "pm":
+                p[1].set_hours(p[1].get_hours() + 12)
+                p[0] = p[1]
+            if p[2] == "am":
+                # print('its am!')
+                p[0] = p[1]
+    elif len(p) == 4:
+        if p[1] == "at":
+            p[0] = p[2]
+    # p[0] = p[2]
+
+
 # TIME - not strictly valid. but should do a single unit of that time
 # NUMBER TIME - not strictly valid. but should work
 # TIME PHRASE -  again not really valid. but should do a single unit of that time
-# TODO - might be able to have 'past phrases' and 'future phrases'
 def p_single_date(p):
     """
+    date : NUMBER
+    date : WORD_NUMBER
+    date : AT NUMBER
+    date : AT WORD_NUMBER
     date : TIME
     date : NUMBER TIME
     date : NUMBER AM
@@ -453,9 +523,13 @@ def p_single_date(p):
     date : PHRASE TIME PHRASE
     """
     if len(p) == 2:
-        p[0] = DateFactory(p[1], 1)
+        params = {
+            "hour": p[1],
+            "minute": 0,
+            "second": 0,
+        }
+        p[0] = DateFactory.create_date(**params)
     elif len(p) == 3:
-        # print('bb', p[1], p[2])
         if isinstance(p[1], int):
             # 5-pm
             if p[2] == "am":
@@ -468,6 +542,8 @@ def p_single_date(p):
                 }
                 p[0] = DateFactory.create_date(**params)
             elif p[2] == "pm":
+                if p[1] < 12:
+                    p[1] += 12
                 params = {
                     "hour": p[1],
                     "minute": 0,
@@ -478,10 +554,21 @@ def p_single_date(p):
                 params = {p[2]: p[1]}
                 p[0] = DateFactory.create_date_with_offsets(**params)  # '3 days'
             return
-        params = {p[2]: 1}  # TODO - prepend offset_ to the key. passing 1 as no number
-        p[0] = DateFactory.create_date_with_offsets(**params)  # 'In a minute'
+        if isinstance(p[2], str):
+            params = {
+                p[2]: 1
+            }  # TODO - prepend offset_ to the key. passing 1 as no number
+            p[0] = DateFactory.create_date_with_offsets(**params)  # 'a minute'
+        else:
+            params = {
+                "hour": p[2],
+                "minute": 0,
+                "second": 0,
+            }
+            p[0] = DateFactory.create_date(**params)  # 'at 4'
+
     elif len(p) == 4:
-        # print("here we are", p[1], p[2], p[3])
+        # print("at-5-pm", p[1], p[2], p[3])
         if p[1] == "at" or p[1] == "@":
             # at-3-am
             if p[3] == "am":
@@ -494,6 +581,8 @@ def p_single_date(p):
                 }
                 p[0] = DateFactory.create_date(**params)
             elif p[3] == "pm":
+                if p[2] < 12:
+                    p[2] += 12
                 params = {
                     "hour": p[2],
                     "minute": 0,
@@ -507,6 +596,37 @@ def p_single_date(p):
         p[0] = DateFactory.create_date_with_offsets(**params)
 
 
+# combines rules test
+def p_twice(p):
+    """
+    date_twice : date date
+    date_twice : date_day date
+    """
+    # print("Parse 2 phrases!", p[1], p[2])
+    # i.e. '(2 days time) (at 4pm)'
+    # i.e. date_day date = 'wednesday @ 5pm'
+
+    now = stDate()
+    d = p[1]
+    d2 = p[2]
+
+    if d2.get_year() != now.get_year():
+        d.set_year(d2.get_year())
+    if d2.get_month() != now.get_month():
+        d.set_month(d2.get_month())
+    if d2.get_date() != now.get_date():
+        d.set_date(d2.get_date())
+    if d2.get_hours() != now.get_hours():
+        d.set_hours(d2.get_hours())
+    if d2.get_minutes() != now.get_minutes():
+        d.set_minutes(d2.get_minutes())
+    if d2.get_seconds() != now.get_seconds():
+        d.set_seconds(d2.get_seconds())
+
+    # not quite right. but almost...
+    p[0] = d  # p[2]  #[p[1], p[2]]
+
+
 # in : PHRASE WORD_NUMBER TIME?? not getting converted
 def p_single_date_in(p):
     """
@@ -601,6 +721,7 @@ def p_single_date_2moro(p):
 def p_single_date_day(p):
     """
     date_day : DAY
+    date_day : ON DAY
     date_day : PHRASE DAY
     date_day : PAST_PHRASE DAY
     """
@@ -618,9 +739,14 @@ def p_single_date_day(p):
         # go forward each day until it matches
         while day_to_find.lower() != d.get_day(to_string=True).lower():
             if p[1] == "last":
-                d.set_date(d.get_date() - 1)
-            elif p[1] == "next":
+                if d.get_date() == 1:
+                    d.set_date(d.get_date() - 2)
+                else:
+                    d.set_date(d.get_date() - 1)
+            elif p[1] == "next" or p[1] == "on":
                 d.set_date(d.get_date() + 1)
+            # else:
+            #     print("an infinite loop?")
 
         p[0] = d
 
@@ -637,11 +763,7 @@ def p_this_or_next_period(p):
             elif p[2] == "year":
                 d.set_year(d.get_year() - 1)
             elif p[2] == "month":
-                # print(d.get_month())
-                d.set_month(
-                    d.get_month()
-                )  # -1)  #??? not sure why this is not -1???????. this must be a bug in set_month?
-                # print(d.get_month())
+                d.set_month(d.get_month() - 1)
             # elif p[2] == "century":
             #     d.set_year(d.get_year() - 100)
         elif p[1] == "next":
@@ -655,7 +777,6 @@ def p_before_yesterday(p):
     date_before_yesterday : THE BEFORE_YESTERDAY
     date_before_yesterday : THE TIME BEFORE_YESTERDAY
     """
-    # print('before_yesterday decteted')
     d = stDate()
     d.set_date(d.get_date() - 2)
     p[0] = d
@@ -683,7 +804,6 @@ def p_single_date_end(p):
     date_end : THE NUMBER DATE_END OF MONTH
     """
     if len(p) == 3:
-        # print('p:', p[1], p[2])
         d = stDate()
         d.set_date(p[1])
         p[0] = d
@@ -691,13 +811,10 @@ def p_single_date_end(p):
         # print('p-:', p[1], p[2], p[3])
         d = stDate()
         d.set_date(p[2])
-        # print(p[1], "the", p[1] == "the")
         if p[1] == "the":  # the-2-nd
             d.set_date(p[2])
         else:  # january-14-th
-            # print('y!', p[1], p[2])
             m = d.get_month_index_by_name(p[1])
-            # print('THE MONTH IS:',m)
             d.set_month(m)
             d.set_date(p[2])
         p[0] = d
@@ -707,15 +824,11 @@ def p_single_date_end(p):
         if p[1] == "on":  # on-the-1-st
             d.set_date(p[3])
         else:  # april-the-1-st
-            # print('i think we are here::' , p[1], p[2], p[3], p[4])
             m = d.get_month_index_by_name(p[1])
             d.set_month(m)
-            # print('THE date is:', p[3])
             d.set_date(p[3])
-            # print('THE date isaaa:', d.get_date())
         p[0] = d
     if len(p) == 6:
-        # print('p++:', p[1], p[2], p[3], p[4], p[5])
         d = stDate()  # the-18-th-of-january
         m = d.get_month_index_by_name(p[5])
         d.set_month(m)
@@ -832,6 +945,17 @@ def replace_short_words(phrase):
     # phrase = re.sub(r"a few\b", "3", phrase)
     # phrase = re.sub(r"\bseveral\b", "7", phrase)
 
+    phrase = phrase.replace("oclock", "")
+    phrase = phrase.replace("o'clock", "")
+
+    phrase = phrase.replace("2moro", "tomorrow")
+    phrase = phrase.replace("2morro", "tomorrow")
+    phrase = phrase.replace("tomorow", "tomorrow")
+
+    # typos
+    phrase = phrase.replace("febuary", "february")
+    phrase = phrase.replace("feburary", "february")
+
     return phrase
 
 
